#!/usr/bin/php
<?php

/*
	THIS SCRIPT GENERATES A POSTFIX ALIAS HASH MAP.
	It is a helper script to simulate Mac OS X Server 10.5's automatic
	group mailing service in Mac OS X Server 10.6
*/

$config = array(
	"dsclHost"=> "localhost",
	"dsclGroupPath"=> "/LDAPv3/127.0.0.1/Groups",
	"dsclGroupExcludes"=>array("workgroup","com.apple.limited_admin","admin","staff"),
);


function strout($str) {
	echo $str."\n";
}

function dsclQuery($query) {
	global $config;
	
	$data=array();
	$result=1;
	exec("dscl ".$config["dsclHost"]." ".$query." 2>/dev/null",&$data,&$result);
	
	if ($result == 0)
		return $data;
	else
		return false;	
}

function array_merge_unique($arr1, $arr2) {

	foreach($arr2 as $item) {
		if (!in_array($item,$arr1)) {
			array_push($arr1,$item);
		}	
	}
	
	return $arr1;
	
}

function readAttributeFromGroup($group,$attr) {
	global $config;
	
	$errorString = "No such key:";
			
	// read attribute
	$tmpResult=dsclQuery("read ".$config["dsclGroupPath"]."/".$group." ".$attr);
	
	if ($tmpResult!=false && ( strpos($tmpResult[0], $errorString) === 0 )!=true) {
		// remove first item and split
		return array_slice(split(" ",$tmpResult[0]),1);
	}
	else
		return array();
}

function getGroups() {
	global $config;
	
	// array to store valid groups
	$groups=array();
	
	// read all groups
	$tmpGroups=dsclQuery("list ".$config["dsclGroupPath"]);
	
	// cycle through groups and get members
	foreach ($tmpGroups as $group) {
		
		//check if in exludes
		if (!in_array($group,$config["dsclGroupExcludes"])) {
			array_push($groups,$group);
		}	
	}
	
	return $groups;
}

function getMembersOfGroup($group) {	
	return readAttributeFromGroup($group,"Member");
}

function getNestedGroupsOfGroup($group) {	
	return readAttributeFromGroup($group,"NestedGroups");
}

function getNameOfGroupFromUID($id) {
	global $config;
	
	$tmpName=dsclQuery("search ".$config["dsclGroupPath"]." GeneratedUID ".$id);
	
	if ($tmpName!=false && count($tmpName)>0) {
		return substr($tmpName[0],0,strpos($tmpName[0],"\t"));	
	}
	else 
		return false;
}

function getMembersOfGroupRec($group) {
	
	$members=getMembersOfGroup($group);
	$nestedGroups=getNestedGroupsOfGroup($group);
	
	if (is_array($nestedGroups) && count($nestedGroups)>0) {
		foreach($nestedGroups as $nestedGroup) {
			$nestedGroupName=getNameOfGroupFromUID($nestedGroup);
			$members=array_merge_unique($members,getMembersOfGroupRec($nestedGroupName));
		}
	}
	
	return $members;	
}

function main($argv) {
	global $config;
	
	// check arguments
	if (count($argv) != 2) {
		strout("Usage: ".$argv[0]." [alias map filename]");
		die();
	}
	
	// try to open file for writing
	$fh=fopen($argv[1],"w");
	
	if(!$fh) {
		strout("Unable to open ".$argv[1]." for writing");  
		die();
	} 
	
	date_default_timezone_set("Europe/Vienna");
	strout("Starting on ".date("d.m.Y @ H:i",time()));
	
	foreach(getGroups() as $group) {
		strout("Processing group ".$group);
		strout(" - Getting all members (even nested)");
			$members=getMembersOfGroupRec($group);
			strout("   ".implode($members,","));
		//writing to file
		fwrite($fh,$group.": ".implode($members,",")."\n");
	}
	
	fclose($fh);
	strout("Aliases file created");
}

// start
main($_SERVER["argv"]);
?>